from jira.client import JIRA
import jira as JRE
from jira.resources import Issue
from jira.exceptions import JIRAError
import sys
import logging
from requests import ConnectionError, RequestException, HTTPError, URLRequired,TooManyRedirects

class JiraModerator(object):
	"""
	Helper class to connect and manipulate the data in Jira
	"""

	client = None
	clientOptions = {'server': 'https://issues.ypg.com'}	#: conneciton options for Jira
	username = "ypgAutoTester"								#: Automation User username (QA Lead access to Jira)
	password = "@ut0t3$t3r"									#: Automation User password
	logger = None 											#: logger to send loggin information to.  Logger comes from pytest test definitions
	project = None

	def __init__(self,logger):
		"""
		Constructor for class 

		Args :
		logger : (logger) instance of a logging object configured in testing project 
				
		"""
		self.logger = logger

	def connect(self):
		"""
		Establishes a connection to our YPG Jira instance and assignes it to self.client

		Returns : (bool) True if we are connected to Jira and False if we are not

		"""

		success = None

		try:
			self.client = JIRA(self.clientOptions,basic_auth=(self.username, self.password))
			self.client.session()
			success = True

		except ConnectionError, e:
			self.logger.error("Error Connection to Jira :")
			self.logger.error(e)
			self.client = None
			success = False
		except HTTPError, e:
			self.logger.error("Error Connection to Jira :")
			self.logger.error(e)
			self.client = None
			success = False

		return success


	def searchForIssue(self,query):
		"""
		Returns a list of issues that were returned by Jira given the query you specified

		Args : (str) A string representation of a JQL query.  anything you can enter in Jira advanced search can be entered here.

		Returns : (ResultList) a list of jira issues
		"""
		try :
			resultList = self.client.search_issues(query)

		except JIRAError, e:
			self.logger.error("Could not search what you are looking for because "+str(email))
			issues = None

		if len(resultList) < 1:
			issues = None
		elif len(resultList) > 1:
			self.logger.warn("Jira found more than one issue with the search %s . You may want to manually verify the automated process updated the correct issue."%query)
			issues = resultList[0]
		else :
			issues = resultList[0]
		return issues

	def confirmVersion(self,projKey,buildVersion):
		"""
		Confirms that the project version supplied actually exists in Jira for the specified project

		Args :
		projKey: (str) the Jira project acronym
		buildVersion : (str) the version of the application you are testing

		Returns : (bool) True if the version was found in Jira.  False if the version was not found
		"""

		try:
			# get all the versions for the specified project from Jira
			project = self.client.project(projKey)
			projVersions = project.versions

			# search for the version we are testing.  Is it in Jira?
			for version in projVersions:

				if str(version.name) == str(buildVersion):
					self.logger.debug("Matched the specidied buildVersion runtime parameter to version in Jira")
					if not version.released and not version.archived:
						self.logger.debug("We are going to start to test build version "+version.name+" for project "+projKey)
						return True
					else:
						self.logger.warn("The buildVersion you are searching for has been released or archived in Jira and is not a valid testable build version")
						return False
		except JIRAError,e:
			self.logger.error("Could not retrieve the projects versions.  Check that the project exists or that Jira is not down")
			return False

	def getRequiredFields(self,projectKey):
		"""
		Fetches a list of fields that require values to open a new issue in a specified project

		Args : (str) The Jira project acronym for the project you are querying

		Returns : (dict) a dictionary containing the required fields and None values for each.  Returns empty dict if search failed.
		"""

		reqFields = {}

		try :
			# Get a list of fields that are required to supply values for so that an issue can be created
			meta = self.client.createmeta(projectKeys=(projectKey),expand=('projects.issuetypes.fields'))
			
			fields = meta['projects'][0]['issuetypes'][0]['fields']
			
			
			for field in fields:
				if fields[field]['required']:
					reqFields[field] = None

		except JIRAError, e:
			self.logger.error("Could not get required fields for Jira project "+projectKey+" because "+str(e))
		except IndexError, e:
			self.logger.error("Could not get required fields for Jira project "+projectKey+" because "+str(e))
			
		return reqFields
		

	def createIssue(self,data):
		"""
		Creates an issue in Jira with the dictionary of data supplied.  Be sure that data contains all required fields before using this.

		Args : (dict) dictionary of required fields and valid values

		Returns : (bool) returns True if issues was created and False if there was a failure
		"""
		success = None

		try:
			newIssue = self.client.create_issue(fields=data)
			success = True

		except JIRAError, e:
			success = False
			self.logger.error("Issue was not created :"+str(e))

		return success

	def reopenBug(self,issue,projKey,version):
		"""
		Transitions a specified jira Bug from any resolved state back to In Review and assigns it to the project lead with comments

		Args : 
		issue : (issue) an issue object that came from Jira.  Use searchForIssue first before reopening issues
		projKey : (str) the Jira project acronym from Jira
		version : (str) the build number currently under test where the bug was rediscovered

		Returns : (bool) returns False if we could not reopen issue and True if we could
		"""
		curState = issue.fields.status.name
		
		transitions = self.client.transitions(issue)

		project = self.client.project(projKey)
		projLead = project.lead.name

		version = version

		comment = "This issue has reoccured in the latest version %s"%version

		try :
			#print transDict
			if curState == "Closed":
				self.client.transition_issue(issue,self.getTransitionId(transitions,"Re-Open"),assignee={'name': projLead})
				self.client.add_comment(issue,comment)
			elif curState == "Ready for QA":
				self.client.transition_issue(issue,self.getTransitionId(transitions,"Back to In Development"),assignee={'name': projLead})
				self.client.add_comment(issue,comment)
			elif curState == "In Testing":
				self.client.transition_issue(issue,self.getTransitionId(transitions,"Fail"),assignee={'name': projLead})
				self.client.add_comment(issue,comment)
			elif curState == "Ready to Deploy":
				self.client.transition_issue(issue,self.getTransitionId(transitions,"Back to In Testing"))
				transitions = self.client.transitions(issue)
				self.client.transition_issue(issue,self.getTransitionId(transitions,"Fail"),assignee={'name': projLead})
				self.client.add_comment(issue,comment)
		except IndexError, e:
			self.logger.error("Could not find a transition to reopen the issue '"+issue.key+"'")
			return False
		except JIRAError, e:
			self.logger.error("Jira returned error when modifying issue '"+issue.key+"' because "+str(e))
			return False

		return True


	def getTransitionId(self,transDict,transName):
		"""
		Fetch the id for a transition's name 

		Args :
		transDict : (dict) a dictionary of transitions fetched from Jira for a given issue
		transName : (str) name of the Jira transition you would like the id for

		Returns : (str) a numeric id associtated to the transition name
		"""

		idDict = [element['id'] for element in transDict if element['name'] == transName]
		return idDict[0]

	def prepareIssueData(self,reqFields,testId,project,summary,description,component,severity,version):
		"""
		Constructs a properly formatted dictionary of data to supply to Jira for opening an issue.  Creates a bug in the specified project
		After construction, it will validate the dictionary by checking if all required fields are filled

		Args : 
		reqFields : (dict) dictionary of jira project required fields.  Construct the dict with getRequiredFields
		testId : (str) the name of the test cases found in the test case's decoreator in python or the name in Spiratest
		project : (str) the Jira project acronym for the project you wish to open a bug inspect
		summary : (str) the summary of the bug you wish to open
		description : (str) the description of the bug you wish to open
		component : (str) the component of the bug you wish to open
		severity : (str) the severity of the bug you wish to open
		version : (str) the affected version of the bug you wish to open

		Returns : (dict) if the dictionary properly complies to all the required fields.  Returns emtpy dict if it does not.

		"""
		desc = "This issue was created by YPG automated Test Case : %(testId)s. \n \n The error is caused when sending the following parameters to the API method in question : \n %(description)s" % {'description':description,'testId':testId}

		reqFields['project'] = {'key':project}
		reqFields['summary'] = summary
		reqFields['description'] = desc
		reqFields['issuetype'] = {'name':'Bug'}
		reqFields['customfield_10411'] = {'value':severity}
		reqFields['components'] = [{'name':component}]
		reqFields['versions'] = [{'name':version}]
		
		# if any of the required fields are not filled erase the dict
		if None in reqFields.values():
			reqFields = {}
		
		return reqFields


# # #create logger
# logger = logging.getLogger('test')
# logger.setLevel(logging.DEBUG)

# # create console handler and set level to debug
# ch = logging.StreamHandler()
# ch.setLevel(logging.DEBUG)

# # create formatter
# formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')

# # add formatter to ch
# ch.setFormatter(formatter)

# #add ch to logger
# logger.addHandler(ch)


# myJira = JiraModerator(logger)
# print myJira.connect()

# resp = myJira.searchForIssue("project = 'Jira Testing'")
# print resp