import os
import json
import jsonschema
import jsonpath
from StringIO import StringIO
from Helper import Helper
from HelperException import FileNotExistException, BadFormatException, NotImplementedException

class JsonHelper(Helper):
    """
    Helper class to manipulate the data in Json format
    """
    _obj = None
    _schema = None
    
    def __init__(self, data, schema=None):
        """
        Constructor for class 

        @param data : the json string to be processed (alternatively, it can the location of the json file)
        @param schema : path to the json schema file
                
        """
        Helper.__init__(self)

        ss = data
        if not data.lstrip().startswith('{'):
            if os.access(data, os.F_OK):
                raise FileNotExistException("Json file path location [%s] does not exist or not accessible " % data[:63])
            with open(data, 'rb') as fd:
                ss = fd.read()
        if not ss.lstrip().startswith('{'):
            raise BadFormatException('Data provided is not a valid json string:\n%s' % ss[:63])
        self._schema = schema
        try:
            self._obj = json.loads(data)
        except Exception as o:
            raise BadFormatException("Json cannot be parsed with error message:\n%s" % o.error_log.last_error)
        

    def validateBySchema(self, schema=None):
        """
        Check if Json string is compliant with schema

        Returns : (bool) whether schema validation is passed 
        """
        if schema:
            self._schema = schema
        if not os.access(self._schema, os.F_OK):
            raise FileNotExistException("Json schema path location [%s] does not exist or not accessible " % self._schema)
        with open(self._schema, 'rb') as fd:
            ss = fd.read()
        if not ss.lstrip().startswith('{'):
            raise BadFormatException('Schema file content is not a valid json string\n%s' % ss)
        raise NotImplementedException('Not fully Implemented')

    
    def getValuesByJSONPath(self, jpath):
        """
        Get all values using JSONPath
        
        Args:
            jpath : JSON-path used to search for values

        For more details about the JSON-Path, please refer to http://goessner.net/articles/JsonPath/
        
        Returns : all matching values from JSONPath search in a list 
        """
        values =  jsonpath.jsonpath(self._obj, jpath)
        return values if values else []


    def getValueByJSONPath(self, jpath):
        """
        Get the first value using JSONPath
        
        Args:
            jpath : JSON-path used to search for values

        For more details about the JSON-Path, please refer to http://goessner.net/articles/JsonPath/
        
        Returns : the first matching value from JSONPath search result 
        """
        values = self.getValuesByJSONPath(jpath)
        return values.split(';')[0] if values else None


    def normalizeJSONPath(self, jpath):
        """
        Get the normalized JSONPath
        
        Args:
            jpath : JSON-path used to fetch normalized path

        For more details about the JSON-Path, please refer to http://goessner.net/articles/JsonPath/
        
        Returns : normalized JSONPath
        """
        paths =  jsonpath.jsonpath(self._obj, jpath, result_type="PATH")
        return paths.split(';')[0] if values else None
    


    def toString(self, pretty_print=True):
        """
        Print Json data in a nice format
        
        Args:
            pretty_print : do you want to be pretty?

        Returns : formatted Json string
        """
        separators = None
        if pretty_print:
            separators = (',', ': ')
        return json.dumps(self._obj, separators=separators)
