import os
from lxml import etree
from StringIO import StringIO
from Helper import Helper
from HelperException import FileNotExistException, BadFormatException, NotImplementedException

class XmlHelper(Helper):
    """
    Helper class to manipulate the data in Xml format
    """
    _schema = None
    _tree = None

    def __init__(self, data, schema=None):
        """
        Constructor for class 

        @param data : the xml string to be processed (alternatively, it can the location of the xml file)
        @param schema : path to the xml schema file
                
        """
        Helper.__init__(self)
        
        self._schema = schema
        ss = data
        if not data.lstrip().startswith('<'):
            if os.access(data, os.F_OK):
                raise FileNotExistException("XML file path location [%s] does not exist or not accessible " % data[:63])
            with open(data, 'rb') as fd:
                ss = fd.read()
        if not ss.lstrip().startswith('<'):
            raise BadFormatException('Data input is not a valid XML string\n%s' % ss[:63])
        try:
            self._tree = etree.parse(StringIO(data))
        except etree.XMLSyntaxError, e:
            raise BadFormatException("XML cannot be parsed with error message:\n%s" % e.error_log.last_error)
        
        
    def validateBySchema(self, schema=None):
        """
        Check if Xml string is compliant with schema
        
        @param schema : path to the xml schema file

        @return (bool) whether schema validation is passed 
        """
        if schema:
            self._schema = schema
        ss = self._schema
        if not self._schema.lstrip().startswith('<'):
            if os.access(self._schema, os.F_OK):
                raise FileNotExistException("Xml schema path location [%s] does not exist or not accessible " % self._schema)
            with open(self._schema, 'rb') as fd:
                ss = fd.read()
        if not ss.lstrip().startswith('<'):
            raise BadFormatException('Schema file content is not a valid XML string\n%s' % ss)
        es = etree.XMLSchema(etree.parse(StringIO(ss)))
        status = es.validate(self._tree)
        return status, es.error_log


    def getRoot(self):
        """
        Get the root element of the xml
        
        @return: root element 
        """
        return self._tree.getroot()
    
    
    def getElementsTextByXPath(self, xpath):
        """
        Get all elements' text values using XPath
        
        @param xpath : Xpath used to search for elements text

        @return: all matching elements' text values from Xpath search result in a list 
        """
        return map(lambda x: x.text, self._tree.xpath(xpath))


    def getElementsByXPath(self, xpath):
        """
        Get all elements using XPath
        
        @param xpath : Xpath used to search for elements

        @return : all matching elements from Xpath search result in a list 
        """
        return self._tree.xpath(xpath)


    def getElementTextByXPath(self, xpath):
        """
        Get the first element text using XPath
        
        @param xpath : Xpath used to search for elements text

        @return : the first matching element text from Xpath search result
        """
        texts = self.getElementsTextByXPath(xpath)
        return texts[0] if texts else None


    def getElementByXPath(self, xpath):
        """
        Get the first element using XPath
        
        @param xpath : Xpath used to search for element

        @return : the first matching element from Xpath search result
        """
        elements = self.getElementsByXPath(xpath)
        return elements[0] if elements else None


    def toString(self, pretty_print=True):
        """
        Print Xml data in a nice format
        
        @param pretty_print : do you want to be pretty?

        @return : formatted xml string
        """
        return etree.tostring(self._tree, pretty_print=pretty_print)
